// ------------------------------------------
// SpySatellite.cs
// ------------------------------------------

datablock TurretData(DeployedSpySatellite) : TurretDamageProfile
{
   shapeFile = "turret_indoor_deployc.dts";
   heatSignature = 0.0;
   deployedObject = true;
   primaryAxis = revzaxis;
   className = DeployedTurret;
   mass = 0.7;
   maxDamage = 10.0;
   destroyedLevel = 10.0;
   disabledLevel = 9.0;
   explosion      = CameraGrenadeExplosion;
   repairRate = 0;
   heatSignature = 0.0;
   deployedObject = true;
   thetaMin = 120;
   thetaMax = 180;
   thetaNull = 180;
   primaryAxis = zaxis;
   isShielded = true;
   energyPerDamagePoint = 1;
   maxEnergy = 1000;
   rechargeRate = 10.0;
   barrel = SpySatelliteBarrel;
   canControl = true;
   canObserve = true;
   cmdCategory = "DSupport";
   cmdIcon = CMDCameraIcon;
   cmdMiniIconName = "commander/MiniIcons/com_camera_grey";
   targetNameTag = 'command';
   targetTypeTag = 'Satellite';
   sensorData = CameraSensorObject;
   sensorRadius = CameraSensorObject.detectRadius;
   firstPersonOnly = true;
   renderWhenDestroyed = false;
   observeParameters = "0.5 4.5 4.5";
   debrisShapeName = "debris_generic_small.dts";
   debris = TurretDebrisSmall;
};

datablock TurretImageData(SpySatelliteBarrel)
{
   shapeFile = "turret_muzzlepoint.dts";
   item      = SpySatelliteBarrel;

   // Turret parameters
   activationMS      = 150;
   deactivateDelayMS = 300;
   thinkTimeMS       = 150;
   degPerSecTheta    = 580;
   degPerSecPhi      = 960;
   attackRadius      = 1;

   // State transitions
   stateName[0]                     = "Activate";
   stateTransitionOnTimeout[0]      = "ActivateReady";
   stateTimeoutValue[0]             = 0.5;
   stateSequence[0]                 = "Activate";

   stateName[1]                     = "ActivateReady";
   stateTransitionOnLoaded[1]       = "Ready";
   stateTransitionOnNoAmmo[1]       = "NoAmmo";

   stateName[2]                     = "Ready";
   stateTransitionOnTriggerDown[2]  = "Fire";

   stateName[3]                     = "Fire";
   stateTransitionOnTimeout[3]      = "Reload";
   stateTimeoutValue[3]             = 0.5;
   stateFire[3]                     = true;
   stateAllowImageChange[3]         = false;
   stateSequence[3]                 = "Fire";
   stateScript[3]                   = "onFire";

   stateName[4]                     = "Reload";
   stateTransitionOnTimeout[4]      = "Ready";
   stateTimeoutValue[4]             = 0.5;
   stateAllowImageChange[4]         = false;
};

function SpySatelliteBarrel::onFire(%data,%obj,%slot)
{
   %range = 5000;
   %rot = rotFromTransform(%obj.getTransform());
   %muzzlePos = %obj.getMuzzlePoint(%slot);
   %muzzleVec = %obj.getMuzzleVector(%slot);
   %endPos    = VectorAdd(%muzzlePos, VectorScale(%muzzleVec, %range));

   %terrain = ContainerRayCast(%muzzlePos, %endPos, $TypeMasks::TerrainObjectType, %obj);
   %message = "";
   if (%terrain)
   {
      %xy = getWords(%terrain, 1, 2);
      %z = getWord(%terrain, 3);
      %pos = %xy SPC %z + %obj.height;
      %mask = ($TypeMasks::VehicleObjectType | $TypeMasks::StaticShapeObjectType | $TypeMasks::ForceFieldObjectType | 
               $TypeMasks::PlayerObjectType | $TypeMasks::TurretObjectType);
      InitContainerRadiusSearch( %pos, 1, %mask );
      %test = containerSearchNext();
      if (%test)
         %message = "Can\'t reposition the satellite. Trajectory Blocked";
      else
      {
         %obj.setTransform(%pos SPC %rot);
      }
   }
   else
      %message = "Trajectory not understood. Reenter.";
   if (%message !$= "")
      messageClient(%obj.client, 'msgBlocked', %message);
}


datablock ShapeBaseImageData(SpySatelliteDeployableImage)
{
   shapeFile = "pack_deploy_sensor_pulse.dts";
   item = SpySatelliteDeployable;
   mountPoint = 1;
   offset = "0 0 0";
   deployed = DeployedSpySatellite;

   stateName[0] = "Idle";
   stateTransitionOnTriggerDown[0] = "Activate";

   stateName[1] = "Activate";
   stateScript[1] = "onActivate";
   stateTransitionOnTriggerUp[1] = "Idle";
   deploySound = SpySatelliteDeploySound;

   maxDepSlope = 40;
   emap = true;
   heatSignature = 0;

   minDeployDis                       =  0.1;
   maxDeployDis                       =  15.0;  //meters from body
};

datablock ItemData(SpySatelliteDeployable)
{
   className = Pack;
   catagory = "Deployables";
   shapeFile = "pack_deploy_sensor_pulse.dts";
   mass = 2.0;
   elasticity = 0.2;
   friction = 0.6;
   pickupRadius = 1;
   rotate = false;
   image = "SpySatelliteDeployableImage";
   pickUpName = "a command satellite pack";

   computeCRC = true;
   emap = true;
};


function SpySatelliteDeployable::onPickup(%this, %obj, %shape, %amount)
{
   // created to prevent console errors
}

function SpySatelliteDeployableImage::testObjectTooClose(%item)
{
   %xy = getWords(%item.surfacePt, 0, 1);
   %z = getWord(%item.surfacePt, 2);
   %z += 200;
   %item.surfacePt = %xy SPC %z;
   %item.surfaceNrm = "0 0 -1";
   %mask =    ($TypeMasks::VehicleObjectType     | $TypeMasks::MoveableObjectType   |
               $TypeMasks::StaticShapeObjectType |
               $TypeMasks::ForceFieldObjectType  | $TypeMasks::ItemObjectType       |
               $TypeMasks::PlayerObjectType      | $TypeMasks::TurretObjectType);
   InitContainerRadiusSearch( %item.surfacePt, $MinDeployDistance, %mask );
   %test = containerSearchNext();
   return %test;
}

function SpySatelliteDeployableImage::onDeploy(%item, %plyr, %slot)
{
   %deplObj = Parent::onDeploy(%item, %plyr, %slot);
   %deplObj.setCloaked(true);
   %deplObj.setPassiveJammed(true);
   %deplObj.height = 200;
   $commandsat[%plyr.team] = %deplObj;
}

function DeployedSpySatellite::onTrigger(%data, %obj, %trigger, %state)
{
   switch (%trigger) {
      case 0:
	   if(%state)
		%obj.setImageTrigger(0, true);
	   else
		%obj.setImageTrigger(0, false);
      case 2:
         if(%state) 
         {
            %obj.getDataBlock().playerDismount(%obj);
         }
   }
   if(%trigger == 5){
	if(%state){
	   %heights = "2 5 10 20 50 100 200 500";
	   for(%i = 0; %i < 8; %i++){
		if(%obj.height == getWord(%heights,%i)){
		   %num = %i;
		   %i = 8;
		}
	   }
	   %newnum = %num + 1;
	   if(%newnum > 7)
		%newnum = 0;
	   %obj.height = getWord(%heights,%newnum);
	   messageClient(%obj.getControllingClient(), 'MsgClient', "\c2Movement height set to "@%obj.height@" meters.");
	}
   }
   if(%trigger == 4){
	if(%state){
	   %client = %obj.getControllingClient();
	   if(%obj.placing == 1){
		attemptbuildingcreation(%obj,%obj.buld);
		return;
	   }
	   if(isObject(%obj.selected)){
		%datablock = %obj.selected.getDatablock();
		%DBname = %datablock.getName();

		%eye = %obj.getMuzzleVector(0);
		%pos = %obj.getMuzzlePoint(0);
		%search = containerRayCast(%pos,vectorAdd(%pos,vectorScale(%eye,2000)), $TypeMasks::StaticShapeObjectType | $TypeMasks::VehicleObjectType, %obj);
		if(%DBname $= "TransTruck"){
		   if(%search){
			%srcpos = posFromRaycast(%search);
		      if(vectorDist(%obj.selected.getPosition(),%srcpos) >= 40){
			   messageClient(%obj.getControllingClient(), 'MsgClient', "\c2Target to far away.");
			   return;
		      }
			%srcobj = firstWord(%search);
			if(%srcobj $= %obj.selected)
			   DropCrate(%obj.selected);
			else if(%srcobj.s[iscrate])
			   loadupobject(%srcobj,%obj.selected);
			else if(%srcobj.getDatablock().getName() $= "TransBoat")
			   loadupobject(%obj.selected,%srcobj);
		   }
		   else {
			%Tsearch = containerRayCast(%pos,vectorAdd(%pos,vectorScale(%eye,2000)), $TypeMasks::TerrainObjectType, %obj);
			if(%Tsearch){
			   %Tpos = posFromRaycast(%Tsearch);
			   truckmovetoPos(%obj.selected,%Tpos);
			}
		   }
		}
		else if(%DBname $= "TransBoat"){
		   if(%search){
			%srcpos = posFromRaycast(%search);
		      if(vectorDist(%obj.selected.getPosition(),%srcpos) >= 40){
			   messageClient(%obj.getControllingClient(), 'MsgClient', "\c2Target to far away.");
			   return;
		      }
			%srcobj = firstWord(%search);
			if(%srcobj $= %obj.selected)
			   accessinv(%srcobj,%client);
			else if(%srcobj.s[iscrate] || %srcobj.getDatablock().getName() $= "TransTruck")
			   loadupobject(%srcobj,%obj.selected);
			else if(%srcobj.s[isfactory] && %srcobj.s[factType] !$= "shipyard")
			   transferStuff(%srcobj,%obj.selected,%client);
			else if(%srcobj.getDatablock().getName() $= "DeployableVehiclePad")
			   transferStuff(%obj.selected,%srcobj,%client);
		   }
		   else {
			%Tsearch = containerRayCast(%pos,vectorAdd(%pos,vectorScale(%eye,2000)), $TypeMasks::WaterObjectType, %obj);
			if(%Tsearch){
			   %Tpos = posFromRaycast(%Tsearch);
			   boatmovetoPos(%obj.selected,%Tpos);
			}
		   }
		}
		if(!%search)
		   return;
		%srcobj = firstWord(%search);
		if(isObject(%srcobj.main))
		   %srcobj = %srcobj.main;
		%srcpos = posFromRaycast(%search);
		%className = %srcObj.getDataBlock().className;

		if(%obj.selected.s[isfactory]){
		   if(!%srcobj.s[usable])
			return;
		   if(vectorDist(%obj.selected.getPosition(),%srcpos) >= 40){
			messageClient(%obj.getControllingClient(), 'MsgClient', "\c2Target to far away.");
			return;
		   }
		   %factType = %obj.selected.s[factType];
		   %srcobjname = %srcobj.getDatablock().getName();
		   if(%obj.selected $= %srcobj)
		      accessFactory(%srcobj,%client);
		   else if((%srcobjname $= "DeployableVehiclePad" && !(%factType $= "Building" || %factType $= "Weapon" || %factType $= "ShipYard")) ||
			     (%srcobjname $= "DeployableVehiclePad2" && %factType $= "ShipYard") ||
			     (%srcobjname $= "TransBoat" && %factType !$= "ShipYard") ||
			     (%srcobj.s[iscrate] == 1 && (%factType $= "Building" || %factType $= "Weapon"))){
			if(isObject(%srcobj.station))
			   %srcobj = %srcobj.station;
			transferStuff(%obj.selected,%srcobj,%client);
		   }
		}
		else if(%obj.selected.isVpad){
		   if(!%srcobj.s[usable])
			return;
		   if(vectorDist(%obj.selected.getPosition(),%srcpos) >= 40){
			messageClient(%obj.getControllingClient(), 'MsgClient', "\c2Target to far away.");
			return;
		   }
		   if(%obj.selected $= %srcobj)
			accessinv(%srcobj,%client);
		   else if(%DBname !$= "DeployableVehiclePad2" && %srcobj.getDatablock().getName() $= "TransBoat")
			transferStuff(%obj.selected,%srcobj,%client);
		   else if(%srcobj.s[isfactory] == 1){
			%factType = %srcobj.s[factType];
		   	if((%DBname $= "DeployableVehiclePad" && !(%factType $= "Building" || %factType $= "Weapon" || %factType $= "ShipYard")) || 
			   (%DBname $= "DeployableVehiclePad2" && %factType $= "ShipYard"))
			   transferStuff(%srcobj,%obj.selected,%client);
		   }
		}
		else if(%obj.selected.s[iscrate] == 1){
		   if(!%srcobj.s[usable])
			return;
		   if(vectorDist(%obj.selected.getPosition(),%srcpos) >= 40){
			messageClient(%obj.getControllingClient(), 'MsgClient', "\c2Target to far away.");
			return;
		   }
		   if(%obj.selected $= %srcobj)
			accessinv(%srcobj,%client);
		   else if(%srcobj.s[iscrate])
			transferStuff(%obj.selected,%srcobj,%client);
		   else if(%srcobj.s[factType] $= "Weapon" || %srcobj.s[factType] $= "Building")
			transferStuff(%srcobj,%obj.selected,%client);
		   else if(%srcobj.s[islocker])
			loadupobject(%srcobj,%obj.selected);
		   else if(%srcobj.getDatablock().getName() $= "TransTruck" || %srcobj.getDatablock().getName() $= "TransBoat")
			loadupobject(%obj.selected,%srcobj);
		}
		else if(%obj.selected.s[islocker] == 1){
		   if(vectorDist(%obj.selected.getPosition(),%srcpos) >= 40){
			messageClient(%obj.getControllingClient(), 'MsgClient', "\c2Target to far away.");
			return;
		   }
		   if(%srcobj.s[iscrate] == 1 || %srcobj.getDatablock().getName() $= "TransBoat" || %srcobj.s[factType] $= "Weapon")
			transferSupplies(%srcobj,%obj.selected,%client);
		}
	   }
	}
   }
   else if (%trigger == 3){ //rightclick select
	if(%state){
	   %eye = %obj.getMuzzleVector(0);
	   %pos = %obj.getMuzzlePoint(0);
	   %search = containerRayCast(%pos,vectorAdd(%pos,vectorScale(%eye,2000)), $TypeMasks::StaticShapeObjectType | $TypeMasks::VehicleObjectType | $TypeMasks::TerrainObjectType, %obj);
	   if(%search){
		%srcobj = firstWord(%search);
		if(%srcobj.s[usable] && %srcobj.team == %obj.team){
		   if(%srcobj.s[isFactory] == 1 && %srcobj.built != 1)
			return;
		   %releaseselected = 1;
		   %newselected = %srcobj;
		}
		else if(isObject(%srcobj.main) && %srcobj.main.built == 1){
		   if(%srcobj.main.s[usable] && %srcobj.team == %obj.team){
		      %releaseselected = 1;
			%newselected = %srcobj.main;
		   }
		}
		else if(%srcobj.getDatablock().numMountPoints > 0){
		   if(%srcobj.team == %obj.team){
			%releaseselected = 1;
			%newselected = %srcobj;
		   }
		   else
			%callTarget = %srcobj;
		}
		else{
		   %srcpos = posFromRaycast(%search);
		   containerRadiusSearch(%srcpos,5,$TypeMasks::PlayerObjectType | $TypeMasks::VehicleObjectType);
		   while ((%TObject = containerSearchNext()) != 0 && %valid != 1){
			if(%TObject.team == %obj.team){
			   %releaseselected = 1;
			   %newselected = %TObject;
			   %valid = 1;
			}
			else if(%srcobj.team != 0 && %srcobj.team !$= "")
			   %callTarget = %TObject;
			else
			   %releaseselected = 1;
		   }
		}
		if(%releaseselected == 1){
		   if(isObject(%obj.selected)){
			if(%obj.selected.getDatablock().className $= "Armor")
			   messageClient(%obj.selected.client, 'MgsClient', "\c2Continue on Soldier.");
			else if(%obj.selected.getDatablock().numMountPoints > 0){
			   for(%i = 0; %i < %obj.selected.getDatablock().numMountPoints; %i++){
				if(%seat[%i] = %obj.selected.getMountNodeObject(%i))
				   messageClient(%seat[%i].client, 'MgsClient', "\c2Continue on Soldier.");
			   }
			}
		   }
		   if(isObject(%newselected)){
			if(%newselected.getDatablock().className $= "Armor"){
			   messageClient(%newselected.client, 'MgsClient', "\c2Listen up soldier.");
			   %obj.selected = %newselected;
			   messageClient(%obj.getControllingClient(), 'MgsClient', "\c2Player selected.");
			}
			else if(%newselected.getDatablock().numMountPoints > 0){
			   for(%i = 0; %i < %newselected.getDatablock().numMountPoints; %i++){
				if(%seat[%i] = %newselected.getMountNodeObject(%i))
				   messageClient(%seat[%i].client, 'MgsClient', "\c2Listen up soldier.");
			   }
			   %obj.selected = %newselected;
			   messageClient(%obj.getControllingClient(), 'MgsClient', "\c2Vehicle selected.");
			}
			else{
			   messageClient(%obj.getControllingClient(), 'MgsClient', "\c2Object selected.");
			   %obj.selected = %newselected;
			}
		   }
		}
		if(isObject(%calltarget)){
		   if(%obj.selected.getDatablock().className $= "Armor"){
			messageClient(%obj.selected.client, 'MgsClient', "\c2Attack this target.");
			%calltarget.commandAttachBeacon();
		   }
		   else if(%obj.selected.getDatablock().numMountPoints > 0){
			for(%i = 0; %i < %obj.selected.getDatablock().numMountPoints; %i++){
			   if(%seat[%i] = %obj.selected.getMountNodeObject(%i)){
				messageClient(%seat[%i].client, 'MgsClient', "\c2Attack this target.");
				%calltarget.commandAttachBeacon();
			   }
			}
		   }
		}
	   }
	}
   }
}

function DeployedSpySatellite::onDestroyed(%this, %obj, %prevState)
{
	if (%obj.isRemoved)
		return;
	if ($Host::InvincibleDeployables != 1 || %obj.damageFailedDecon) {
		%obj.isRemoved = true;
		$TeamDeployedCount[%obj.team, SpySatelliteDeployable]--;
		remDSurface(%obj);
		%obj.schedule(500, delete);
	}
	Parent::onDestroyed(%this, %obj, %prevState);
}

function SimObject::CommandAttachBeacon(%obj){
   if(isObject(%obj.enemyBeacon))
	%obj.enemyBeacon.delete();
   %beacon = new BeaconObject(){
      datablock = BomberBeacon;
   };
   if (%obj.team == 1)
      %team = 2;
   else
      %team = 1;
   %beacon.team = %team;
   %beacon.owner = %target;
   %beacon.setTarget(%team);
   %obj.mountObject(%beacon, 4);
   %obj.enemyBeacon = %beacon;
   MissionCleanup.add(%beacon);
   %beacon.setBeaconType(enemy);
}